/* ice16f84.h contains PIC16F84 specific stuff.... */

#define instr_show 1		/* prints an instruction at cursor position	*/
#define instr_exec 2		/* simulates an instruction			*/

#define INDIR	0x00
#define TMR0	0x01
#define OPTION	0x81
#define PCL	0x02
#define STATUS	0x03
#define FSR	0x04
#define PORTA	0x05
#define PORTB	0x06
#define TRISA	0x85
#define TRISB	0x86
#define EEDATA	0x08
#define EECON1	0x88
#define EEADR	0x09
#define EECON2	0x89
#define PCLATH	0x0A
#define INTCON	0x0B

int power2[]={1,2,4,8,16,32,64,128,256};

int	olda, oldb;		/* previous state of I/O ports */
int	oldeecon2;		/* previous state of EECON2 register */
int	config;			/* configuration word */
int	progmem[4096];		/* program memory */
int	eeprom[512];		/* E-eprom data memory */
int	regfile[512];		/* SFR and data memory */
int	stack[7];		/* Stack */
int	breakpoint[32];		/* Breakpoints */
int	w;			/* W register */
int	pc;			/* Program counter */
int	sp;			/* stack pointer */
int 	prescaler;		/* prescaler for Timer0 */

int timertick(void);
void checkportint(void);
void clearshowwindow(void);
int trisa(int status, int command);
int trisb(int status, int command);
int porta(int status, int command);
int portb(int status, int command);

void printinstr(char *instr, int opcode)
	{
	cprintf("%s %0.2X,", instr, opcode&0x7F);
	if(opcode&0x80)
		cprintf("F");
	else
		cprintf("W");
	}

void printlit(char *instr, int opcode)
	{
	cprintf("%s %0.2X", instr, opcode&0xFF);
	if (isprint(opcode&0xFF))
		cprintf(" { %c }", opcode&0xFF);
	}

void clockpulse(void)
	{
	timertick();
	checkportint();
	if ((regfile[INTCON]&0x89)==0x89 || (regfile[INTCON]&0x92)==0x92 || (regfile[INTCON]&0xA4)==0xA4 || ((regfile[INTCON]&0xC0)==0xC0 && (regfile[EECON1]&0x10)==0x10))
		{
                stack[sp]=pc;
                sp++;
                sp&=0x07;
		pc=0x0004;
		regfile[INTCON]&=0x7F;
		}
	}

int getreg(int opcode)			/* returns the value of regfile[] where the register is provided in the opcode */
	{
	switch(opcode&0x7F)
		{
		case INDIR:
			{
			if (regfile[FSR])  /* read indirect through reg 5 */
				{
				regfile[INDIR]=getreg(regfile[FSR]);
				return regfile[INDIR];
				}
			else		  /* read 00 indirect returns 00 */
				return regfile[INDIR];
			break;
			}
		case PORTA:	/* or TRISA */
			{
			if (regfile[STATUS]&0x20)
				{
				regfile[TRISA]=trisa('r', 0) & 0x1F;
				return regfile[TRISA];
				}
			else
				{
				regfile[PORTA]=porta('r', 0) & 0x1F;
				return regfile[PORTA];
				}
			break;
			}
		case PORTB:	/* or TRISB */
			{
			if (regfile[STATUS]&0x20)
				{
				regfile[TRISB]=trisb('r', 0);
				return regfile[TRISB];
				}
			else
				{
				regfile[PORTB]=portb('r', 0);
				return regfile[PORTB];
				}
			break;
			}
		case EEDATA:	/* or EECON1 */
			{
			if(regfile[STATUS]&0x20)
				return regfile[EECON1]&0x1F;
			else
				return regfile[EEDATA];
			break;
			}
		case EEADR:	/* or EECON2 */
			{
			if(regfile[STATUS]&0x20)
				return 0x00;
			else
				return regfile[EEADR];
			break;
			}
		default:
			{
			if(regfile[STATUS]&0x20 && ((opcode&0x7F)==0x01 || (opcode&0x7F)==0x08 || ((opcode&0x7F)==0x09)))
				return regfile[0x80 | (opcode&0x7F)];
        		else
                		return regfile[opcode&0x7F];
			break;
			}
		}
	}

void setreg(int opcode, int value)	/* writes the value into the regfile[] array where the register is provided in the opcode */
	{
	int reg;
	value&=0xFF;
        switch(opcode&0x7F)
                {
		case INDIR:
			{	/* only write when not reg 00 selected in FSR */
			if (regfile[FSR])
				{
				setreg(regfile[FSR], value);
 				regfile[INDIR]=value;
				}
			break;
			}
				
		case TMR0:
			{
                        if (regfile[STATUS]&0x20)
                                {
                                regfile[OPTION]=value;
                                }
                        else
                                {
				regfile[TMR0]=value;
				prescaler=256;
				}
			break;
			}
                case PORTA:	/* or TRISA */
                        {
                        if (regfile[STATUS]&0x20)
				{
                                trisa('w', value);
				regfile[TRISA]=value;
				}
                        else
				{
                                porta('w', value);
				regfile[PORTA]=value;
				}
                        break;
                        }
                case PORTB:	/* or TRISB */
                        {
                        if (regfile[STATUS]&0x20)	
				{
                                trisb('w', value);
				regfile[TRISB]=value;
				}
                        else
				{
                                portb('w', value);
				regfile[PORTB]=value;
				}
                        break;
                        }
		case EEDATA:	/* or EECON1 */
			{
			if (regfile[STATUS]&0x20)
				{
				regfile[EECON1]=(regfile[EECON1]&0x03)|value;
				if (regfile[EECON1]&0x01)		/* Eeprom read? */
					{
					regfile[EEDATA] = eeprom[regfile[EEADR]];
					regfile[EECON1]&=0xFE;
					}
				if (((regfile[EECON1]&0x06)==0x06) && (regfile[EECON2] == 0xAA) && (oldeecon2 == 0x55))		/* Eeprom write? */
					{
					eeprom[regfile[EEADR]]=regfile[EEDATA];
					oldeecon2=0;
					regfile[EECON1]&=0xFD;
					regfile[EECON1]|=0x10;
					}
				}
			else
				regfile[EEDATA]=value;
			break;
			}
		case EEADR:	/* or EECON2 */
			{
			if (regfile[STATUS]&0x20)
				{
				oldeecon2=regfile[EECON2];
				regfile[EECON2]=value;
				}
			else
				regfile[EEADR]=value&0x3F;
			break;
			}
                default:
                        {
			if(regfile[STATUS]&0x20 && ((opcode&0x7F)==0x08 || ((opcode&0x7F)==0x09)))
                		reg = 0x80 | (opcode&0x7F);
        		else
                		reg = opcode&0x7F;
			regfile[reg] = value;
			break;
			}
		}
	}


void opcode_addwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("ADDWF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                pc++;
                clockpulse();
                i=getreg(opcode)+w;
		if ((getreg(opcode)&0x0F)+(w&0x0F) > 0x0F)
			regfile[STATUS]|=0x02;
		else
			regfile[STATUS]&=0xFD;
		if(i>255)
			{
			regfile[STATUS]|=0x01;
			i&=0xFF;
			}
		else
			regfile[STATUS]&=0xFE;
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (i)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_andwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("ANDWF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=getreg(opcode)&w;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (i)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_clrf(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("CLRF %0.2X", opcode&0x7F);
		}
	if (command & instr_exec)
		{
		setreg(opcode,0);
		regfile[STATUS]|=0x04;
		pc++;
		clockpulse();
		}
	}

void opcode_clrw(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("CLRW");
		}
	if (command & instr_exec)
		{
		w=0;
		regfile[STATUS]|=0x04;
		pc++;
		clockpulse();
		}
	}

void opcode_comf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("COMF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=getreg(opcode)^0xFF;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_decf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("DECF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=(getreg(opcode)-1)&0xFF;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
                }
	}

void opcode_decfsz(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("DECFSZ", opcode);
		}
	if (command & instr_exec)
		{
		int i;
		i=(getreg(opcode)-1)&0xFF;
		pc++;
		clockpulse();
		if (!i) 
			{
			pc++;
			clockpulse();
			}
		if (opcode&0x80)
			setreg(opcode,i);
		else
			w=i;
		}
	}

void opcode_incf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("INCF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=(getreg(opcode)+1)&0xFF;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_incfsz(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("INCFSZ", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=(getreg(opcode)+1)&0xFF;
                pc++;
                clockpulse();
                if (!i)
                        {
                        pc++;
                        clockpulse();
                        }
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;

		}
	}

void opcode_iorwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("IORWF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=getreg(opcode)|w;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
}
	}

void opcode_movf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("MOVF", opcode);
		}
	if (command & instr_exec)
		{
                int i;
                i=getreg(opcode);
                pc++;
                clockpulse();
                if (!i)
                        regfile[STATUS]|=0x04;
		else
			regfile[STATUS]&=0xFB;
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
		}
	}

void opcode_movwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("MOVWF %0.2X", opcode&0x7F);
		}
	if (command & instr_exec)
		{
		setreg(opcode,w);
		pc++;
		clockpulse();
		}
	}

void opcode_nop(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("NOP");
		}
	if (command & instr_exec)
		{
		pc++;
		clockpulse();
		}
	}

void opcode_rlf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("RLF", opcode);
		}
	if (command & instr_exec)
		{
		int i,c;
		c=regfile[STATUS]&0x01;
                i=getreg(opcode);
		i=i<<1;
		if(i&0x100)
			regfile[STATUS]|=0x01;
		else
			regfile[STATUS]&=0xFE;
		i=(i&0xFE)+c;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
		}
	}

void opcode_rrf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("RRF", opcode);
		}
	if (command & instr_exec)
		{
                int i,c;
                c=regfile[STATUS]&0x01;
                i=getreg(opcode);
		if(i&0x01)
			regfile[STATUS]|=0x01;
		else
			regfile[STATUS]&=0xFE;
                i=i>>1;
                i=(i&0x7F)+c*0x80;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
		}
	}

void opcode_subwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("SUBWF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                if ((getreg(opcode)&0x0F)-(w&0x0F) < 0x00)
                        regfile[STATUS]&=0xFD;
                else
                        regfile[STATUS]|=0x02;
                i=getreg(opcode)-w;
                if(i<0)
                        {
                        regfile[STATUS]&=0xFE;
                        i&=0xFF;
                        }
		else
			regfile[STATUS]|=0x01;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_swapf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("SWAPF", opcode);
		}
	if (command & instr_exec)
		{
		int i,n;
                i=getreg(opcode)&0x0F;
		n=getreg(opcode)&0xF0;
		i=i<<4;
		n=n>>4;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i+n);
                else
                        w=i+n;
		}
	}

void opcode_xorwf(int opcode, int command)
	{
	if (command & instr_show)
		{
		printinstr("XORWF", opcode);
		}
	if (command & instr_exec)
		{
		int i;
                i=getreg(opcode)^w;
                pc++;
                clockpulse();
                if (opcode&0x80)
                        setreg(opcode,i);
                else
                        w=i;
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_bcf(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("BCF %0.2X,%d", opcode&0x7F, (opcode&0x0380)>>7);
		}
	if (command & instr_exec)
		{
		int b;
		b=power2[(opcode&0x380)>>7];
		b^=0xFF;
		setreg(opcode, getreg(opcode)&b);
		pc++;
		clockpulse();
		}
	}

void opcode_bsf(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("BSF %0.2X,%d", opcode&0x7F, (opcode&0x0380)>>7);
		}
	if (command & instr_exec)
		{
		int b;
		b=power2[(opcode&0x380)>>7];
		setreg(opcode, getreg(opcode)|b);
		pc++;
		clockpulse();
		}
	}

void opcode_btfsc(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("BTFSC %0.2X,%d", opcode&0x7F, (opcode&0x0380)>>7);
		}
	if (command & instr_exec)
		{
                int b;
                b=power2[(opcode&0x380)>>7];
                clockpulse();
                pc++;
		if(!getreg(opcode)&b)
			{
			pc++;
			clockpulse();
			}
		}
	}

void opcode_btfss(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("BTFSS %0.2X,%d", opcode&0x7F, (opcode&0x0380)>>7);
		}
	if (command & instr_exec)
		{
                int b;
                b=power2[(opcode&0x380)>>7];
                pc++;
                clockpulse();
                if(getreg(opcode)&b)
                        {
                        pc++;
                        clockpulse();
                        }
		}
	}

void opcode_addlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("ADDLW", opcode);
		}
	if (command & instr_exec)
		{
		if ((opcode&0x0F)+(w&0x0F) > 0x0F)	/* set Digit Carry */
                        regfile[STATUS]|=0x02;
                else
                        regfile[STATUS]&=0xFD;
		w+=(opcode&0xFF);
                if(w>255)				/* set Carry flag */
                        {
			regfile[STATUS]|=0x01;
                        w&=0xFF;
                        }
		else
			regfile[STATUS]&=0xFE;
                if (w)					/* set Zero flag */
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		pc++;
		clockpulse();
		}
	}

void opcode_andlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("ANDLW", opcode);
		}
	if (command & instr_exec)
		{
                w&=(opcode&0xFF);
                pc++;
                clockpulse();
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_call(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("CALL %0.4X", opcode&0x07FF);
		}
	if (command & instr_exec)
		{
		stack[sp]=pc+1;
		sp++;
		sp&=0x07;
		pc=(opcode&0x07FF)+(regfile[PCLATH]&0x18<<9);
		clockpulse();
		clockpulse();
		}
	}

void opcode_clrwdt(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("CLRWDT");
		}
	if (command & instr_exec)
		{
		/* */
		}
	}

void opcode_goto(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("GOTO %0.4X", opcode&0x07FF);
		}
	if (command & instr_exec)
		{
		pc=(opcode&0x07FF)+(regfile[PCLATH]&0x18<<9);
		clockpulse();
		clockpulse();
		}
	}

void opcode_iorlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("IORLW", opcode);
		}
	if (command & instr_exec)
		{
                w|=(opcode&0xFF);
                pc++;
                clockpulse();
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_movlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("MOVLW", opcode);
		}
	if (command & instr_exec)
		{
		w=opcode&0xFF;
		pc++;
		clockpulse();
		}
	}

void opcode_retfie(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("RETFIE");
		}
	if (command & instr_exec)
		{
		sp--;
		sp&=0x07;
		pc=stack[sp];
		regfile[INTCON]|=0x80;
		clockpulse();
		clockpulse();
		}
	}

void opcode_retlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("RETLW", opcode);
		}
	if (command & instr_exec)
		{
		sp--;
                sp&=0x07;
		w=opcode&0xFF;
                pc=stack[sp];
                clockpulse();
                clockpulse();
		}
	}

void opcode_return(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("RETURN");
		}
	if (command & instr_exec)
		{
		sp--;
		sp&=0x07;
		pc=stack[sp];
		clockpulse();
		clockpulse();
		}
	}

void opcode_sleep(int opcode, int command)
	{
	if (command & instr_show)
		{
		cprintf("SLEEP");
		}
	if (command & instr_exec)
		{
		/* */
		}
	}

void opcode_sublw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("SUBLW", opcode);
		}
	if (command & instr_exec)
		{
                if ((opcode&0x0F)-(w&0x0F) < 0x00)
                        regfile[STATUS]&=0xFD;
                else
                        regfile[STATUS]|=0x02;
		w=(opcode&0xFF)-w;
                if(w<0)
                        {
                        regfile[STATUS]&=0xFE;
                        w&=0xFF;
                        }
		else
			regfile[STATUS]|=0x01;
                pc++;
                clockpulse();
                if (w)
                        regfile[STATUS]&=0xFB;
                else
                        regfile[STATUS]|=0x04;
		}
	}

void opcode_xorlw(int opcode, int command)
	{
	if (command & instr_show)
		{
		printlit("XORLW", opcode);
		}
	if (command & instr_exec)
		{
                pc++;
                clockpulse();
		w^=(opcode&0xFF);
                if (!w)
                        regfile[STATUS]|=0x04;
                else
                        regfile[STATUS]&=0xFB;
		}
	}

void showinstr(int address, int x, int y)
	{
	int opcode;
	opcode=progmem[address&0x03FF];
        gotoxy(x,y);
        cprintf("                         ");
        gotoxy(x,y);
        cprintf("%0.4X: ", address&0x03FF);
	if (config & 0x3FF0)
		{
		cprintf("%0.4X ", opcode);
		if(((opcode&0x3F00)^0x0700) == 0)	opcode_addwf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0500) == 0)	opcode_andwf(opcode, instr_show);
		if(((opcode&0x3F80)^0x0180) == 0)	opcode_clrf(opcode, instr_show);
		if(((opcode&0x3F80)^0x0100) == 0)	opcode_clrw(opcode, instr_show);
		if(((opcode&0x3F00)^0x0900) == 0)	opcode_comf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0300) == 0)	opcode_decf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0B00) == 0)	opcode_decfsz(opcode, instr_show);
		if(((opcode&0x3F00)^0x0A00) == 0)	opcode_incf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0F00) == 0)	opcode_incfsz(opcode, instr_show);
		if(((opcode&0x3F00)^0x0400) == 0)	opcode_iorwf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0800) == 0)	opcode_movf(opcode, instr_show);
		if(((opcode&0x3F80)^0x0080) == 0)	opcode_movwf(opcode, instr_show);
		if(((opcode&0x3F9F)^0x0000) == 0)	opcode_nop(opcode, instr_show);
		if(((opcode&0x3F00)^0x0D00) == 0)	opcode_rlf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0C00) == 0)	opcode_rrf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0200) == 0)	opcode_subwf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0E00) == 0)	opcode_swapf(opcode, instr_show);
		if(((opcode&0x3F00)^0x0600) == 0)	opcode_xorwf(opcode, instr_show);
		if(((opcode&0x3C00)^0x1000) == 0)	opcode_bcf(opcode, instr_show);
		if(((opcode&0x3C00)^0x1400) == 0)	opcode_bsf(opcode, instr_show);
		if(((opcode&0x3C00)^0x1800) == 0)	opcode_btfsc(opcode, instr_show);
		if(((opcode&0x3C00)^0x1C00) == 0)	opcode_btfss(opcode, instr_show);
		if(((opcode&0x3E00)^0x3E00) == 0)	opcode_addlw(opcode, instr_show);
		if(((opcode&0x3F00)^0x3900) == 0)	opcode_andlw(opcode, instr_show);
		if(((opcode&0x3800)^0x2000) == 0)	opcode_call(opcode, instr_show);
		if(((opcode&0x3FFF)^0x0064) == 0)	opcode_clrwdt(opcode, instr_show);
		if(((opcode&0x3800)^0x2800) == 0)	opcode_goto(opcode, instr_show);
		if(((opcode&0x3F00)^0x3800) == 0)	opcode_iorlw(opcode, instr_show);
		if(((opcode&0x3C00)^0x3000) == 0)	opcode_movlw(opcode, instr_show);
		if(((opcode&0x3FFF)^0x0009) == 0)	opcode_retfie(opcode, instr_show);
		if(((opcode&0x3C00)^0x3400) == 0)	opcode_retlw(opcode, instr_show);
		if(((opcode&0x3FFF)^0x0008) == 0)	opcode_return(opcode, instr_show);
		if(((opcode&0x3FFF)^0x0063) == 0)	opcode_sleep(opcode, instr_show);
		if(((opcode&0x3E00)^0x3C00) == 0)	opcode_sublw(opcode, instr_show);
		if(((opcode&0x3F00)^0x3A00) == 0)	opcode_xorlw(opcode, instr_show);
		}
	else
		cprintf("Code is protected");
	}

void execinstr(int address)
	{
	int opcode;
	opcode=progmem[address&0x03FF];
	if(((opcode&0x3F00)^0x0700) == 0)	opcode_addwf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0500) == 0)	opcode_andwf(opcode, instr_exec);
	if(((opcode&0x3F80)^0x0180) == 0)	opcode_clrf(opcode, instr_exec);
	if(((opcode&0x3F80)^0x0100) == 0)	opcode_clrw(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0900) == 0)	opcode_comf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0300) == 0)	opcode_decf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0B00) == 0)	opcode_decfsz(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0A00) == 0)	opcode_incf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0F00) == 0)	opcode_incfsz(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0400) == 0)	opcode_iorwf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0800) == 0)	opcode_movf(opcode, instr_exec);
	if(((opcode&0x3F80)^0x0080) == 0)	opcode_movwf(opcode, instr_exec);
	if(((opcode&0x3F9F)^0x0000) == 0)	opcode_nop(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0D00) == 0)	opcode_rlf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0C00) == 0)	opcode_rrf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0200) == 0)	opcode_subwf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0E00) == 0)	opcode_swapf(opcode, instr_exec);
	if(((opcode&0x3F00)^0x0600) == 0)	opcode_xorwf(opcode, instr_exec);
	if(((opcode&0x3C00)^0x1000) == 0)	opcode_bcf(opcode, instr_exec);
	if(((opcode&0x3C00)^0x1400) == 0)	opcode_bsf(opcode, instr_exec);
	if(((opcode&0x3C00)^0x1800) == 0)	opcode_btfsc(opcode, instr_exec);
	if(((opcode&0x3C00)^0x1C00) == 0)	opcode_btfss(opcode, instr_exec);
	if(((opcode&0x3E00)^0x3E00) == 0)	opcode_addlw(opcode, instr_exec);
	if(((opcode&0x3F00)^0x3900) == 0)	opcode_andlw(opcode, instr_exec);
	if(((opcode&0x3800)^0x2000) == 0)	opcode_call(opcode, instr_exec);
	if(((opcode&0x3FFF)^0x0064) == 0)	opcode_clrwdt(opcode, instr_exec);
	if(((opcode&0x3800)^0x2800) == 0)	opcode_goto(opcode, instr_exec);
	if(((opcode&0x3F00)^0x3800) == 0)	opcode_iorlw(opcode, instr_exec);
	if(((opcode&0x3C00)^0x3000) == 0)	opcode_movlw(opcode, instr_exec);
	if(((opcode&0x3FFF)^0x0009) == 0)	opcode_retfie(opcode, instr_exec);
	if(((opcode&0x3C00)^0x3400) == 0)	opcode_retlw(opcode, instr_exec);
	if(((opcode&0x3FFF)^0x0008) == 0)	opcode_return(opcode, instr_exec);
	if(((opcode&0x3FFF)^0x0063) == 0)	opcode_sleep(opcode, instr_exec);
	if(((opcode&0x3E00)^0x3C00) == 0)	opcode_sublw(opcode, instr_exec);
	if(((opcode&0x3F00)^0x3A00) == 0)	opcode_xorlw(opcode, instr_exec);
	}

void pic_power_on(void)
	{
	regfile[TRISA]=0x1F;
	regfile[TRISB]=0xFF;
	regfile[STATUS]=0x18;
	regfile[OPTION]=0xFF;
	regfile[PCL]=0x00;
	regfile[PCLATH]=0x00;
	pc=sp=0;
	prescaler=256;
	}

void pic_reset(int ask)
	{
	int reset=0;
	if (ask)
		{
		clearshowwindow();
		gotoxy(25,20);
		cprintf("Do you really want to reset the PIC (y/n) ?");
		reset=toupper(readkey());
		while (reset != 'Y' && reset != 'N')
			reset=toupper(readkey());
		}
	if (reset=='Y')
		{
		pc=sp=0;
	        regfile[STATUS]&=0x1F;
        	regfile[OPTION]=0xFF;
	        regfile[PCL]=0x00;
		regfile[PCLATH]=0x00;
 		regfile[INTCON]&=0x01;
	        regfile[TRISA]=0x1F;
	        regfile[TRISB]=0xFF;
		prescaler=256;
		}
	}

int iscall(int address)
	{
	if(((progmem[address]&0x3800)^0x2000) == 0)
		return 1;
	else
		return 0;
	}
	
int timertick(void)
	{	/* returns 1 if timer interrupt occurred */
	if (((regfile[OPTION]&0x20)==0) || ((regfile[OPTION]&0x30)==0x20 && (getreg(PORTA)&0x10)==0 && (olda&0x10)==0x10) || ((regfile[OPTION]&0x30)==0x30 && (getreg(PORTA)&0x10)==0x10 && (olda&0x10)==0))
		{
		if (regfile[OPTION]&0x08)
			{	 /* no prescaler for timer */
			regfile[TMR0]++;
			}
		else
			{	/* prescaler for timer */
			prescaler--;
			if (prescaler==0)
				{
				regfile[TMR0]++;
				prescaler=power2[1+(regfile[OPTION]&0x07)];
				}
			}
		}
	if (regfile[TMR0]==256)
		{
		regfile[TMR0]=0;
		regfile[INTCON]|=0x04;
		if (regfile[INTCON]&0x20) return 1;
		}
	return 0;
	}

void checkportint(void)
	{
	int portstat;
	portstat=porta('r',0);
	if ( (olda & 0x1F) != (portstat & 0x1F) )
		regfile[INTCON]|=0x01;
	portstat=portb('r',0);
	if ( (((regfile[INTCON] & 0x40) == 0x40) && ((oldb & 0x01) == 0) && ((portstat &0x01) == 1)) || (((regfile[INTCON] & 0x40) == 0x00) && ((oldb & 0x01) == 1) && ((portstat & 0x01) ==0)))
		regfile[INTCON]|=0x02;
	}
